package java.nio.channels.spi;

import java.nio.channels.Channel;
import java.nio.channels.SocketChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.Pipe;
import java.net.ProtocolFamily;
import java.io.IOException;
import java.nio.channels.DatagramChannel;
import java.security.AccessController;
import sun.nio.ch.DefaultSelectorProvider;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.util.ServiceLoader;
import java.util.ServiceConfigurationError;

public abstract class SelectorProvider
{
    private static final Object lock;
    private static SelectorProvider provider;
    
    protected SelectorProvider() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("selectorProvider"));
        }
    }
    
    private static boolean loadProviderFromProperty() {
        String property = System.getProperty("java.nio.channels.spi.SelectorProvider");
        if (property == null) {
            return false;
        }
        try {
            SelectorProvider.provider = (SelectorProvider)Class.forName(property, true, ClassLoader.getSystemClassLoader()).newInstance();
            return true;
        }
        catch (ClassNotFoundException cause) {
            throw new ServiceConfigurationError(null, cause);
        }
        catch (IllegalAccessException cause2) {
            throw new ServiceConfigurationError(null, cause2);
        }
        catch (InstantiationException cause3) {
            throw new ServiceConfigurationError(null, cause3);
        }
        catch (SecurityException cause4) {
            throw new ServiceConfigurationError(null, cause4);
        }
    }
    
    private static boolean loadProviderAsService() {
        Iterator<SelectorProvider> iterator = ServiceLoader.load(SelectorProvider.class, ClassLoader.getSystemClassLoader()).iterator();
        Label_0014: {
            try {
                if (!iterator.hasNext()) {
                    return false;
                }
                SelectorProvider.provider = (SelectorProvider)iterator.next();
                return true;
            }
            catch (ServiceConfigurationError serviceConfigurationError) {
                if ((serviceConfigurationError.getCause() instanceof SecurityException)) {
                    continue;
                }
                throw serviceConfigurationError;
            }
        }
    }
    
    public static SelectorProvider provider() {
        synchronized (SelectorProvider.lock) {
            if (SelectorProvider.provider != null) {
                return SelectorProvider.provider;
            }
            return (SelectorProvider)AccessController.doPrivileged(new PrivilegedAction<SelectorProvider>() {
                @Override
                public SelectorProvider run() {
                    if (SelectorProvider.access$000()) {
                        return SelectorProvider.provider;
                    }
                    if (SelectorProvider.access$200()) {
                        return SelectorProvider.provider;
                    }
                    SelectorProvider.provider = DefaultSelectorProvider.create();
                    return SelectorProvider.provider;
                }
            });
        }
    }
    
    public abstract DatagramChannel openDatagramChannel() throws IOException;
    
    public abstract DatagramChannel openDatagramChannel(ProtocolFamily p0) throws IOException;
    
    public abstract Pipe openPipe() throws IOException;
    
    public abstract AbstractSelector openSelector() throws IOException;
    
    public abstract ServerSocketChannel openServerSocketChannel() throws IOException;
    
    public abstract SocketChannel openSocketChannel() throws IOException;
    
    public Channel inheritedChannel() throws IOException {
        return null;
    }
    
    static boolean access$000() {
        return loadProviderFromProperty();
    }
    
    static boolean access$200() {
        return loadProviderAsService();
    }
    
    static {
        lock = new Object();
        SelectorProvider.provider = null;
    }
}
